import re

FAREWELL_MATCHER = re.compile("|".join([
    r'all the best',
    r'cheers',
    r'degards',
    r'have a nice weekend',
    r'many thanks( (&|and)? kind regards)?',
    r'mit freundlichen grüßen',
    r'sincerely',
    r'speak to you soon',
    r'(thanks|thank you)( for your understanding)?( (&|and))?( (best|kind))?( regards)?( from)?',
    r'(with )?((warm(est)?|kind|best) )?(regards|wishes)',
    r'with my best',
    r'yours',
    r'yours( (faithfully|sincerely|truly|))?',
    r'looking forward to (meeting|hearing from) you',
    r'ta',
    r'best',
    r'take care',
    r'thank you, best wishes',
    r'have a (good|great) day',
    r'kindest regards',
    r'hope to hear from you soon',
    r'thx',
    r'tks',
    r'rgds',
    r'(i )?look forward to catching up'
]))

SIGNATURE_MATCHER = re.compile("|".join([
  r'get outlook for \w+',
  r'sent from \w+( \w+)? (on|for) \w+( \w+)?'
]), flags=re.MULTILINE)

LEGAL_MATCHER = re.compile("|".join([
  r'any and all information contained in this email and/or attachments thereto is confidential',
  r'this transmission is intended solely for the addressee and contains confidential information.',
  r'the content of this email and (its|any) attachments (\(\“the email\”\))? (is|are) confidential',
  r'this message may contain information that is confidential or privileged',
  
  r'sent from my',
  r'this message is intended for the addressee named and can contain',
  r'disclaimer: this email and any attachments are intended only for the addressee named and may contain confidential'
  r'-+ forwarded message -+',
  r'DISCLAIMER: this email and any attachments are intended only for the addressee'
]), flags=re.MULTILINE)



import re
from syntok import segmenter

from neon_goby import constants
from neon_goby.matchers import FAREWELL_MATCHER, LEGAL_MATCHER, SIGNATURE_MATCHER


NAME_ALLOWANCE = 2 # reduced as its now words instead of characters

def split(text):
    """
    Split input text into sentences/headers/footers, remove greetings and all
    text after farewell.
    """
    paragraphs = text.splitlines()
    tokenised_texts = [segmenter.process(paragraph) \
        for paragraph in paragraphs]
    all_sentences = []
    for tokenised_text in tokenised_texts:
        sentences = [''.join(map(str, sentence)).strip() \
            for paragraph in tokenised_text for sentence in paragraph]
        all_sentences.extend(sentences)
    useful_sentences = []
    for i, sentence in enumerate(all_sentences):
        if i < 2:
            sentence = _strip_greeting(sentence)
        if i > 1:
            # if there is anything after the first comma, its probably
            # not a farewell
            # TODO(hii): Use POS tagging for word after greeting 
            # to figure out if its indeed a farewell
            if len(','.join(sentence.split(",")[1:]).strip()) == 0:
                if  _contains_valid_farewell(sentence.lower().strip()):
                    break
            
            # strip signatures
            if SIGNATURE_MATCHER.match(sentence.lower().strip()):
                break

            # strip legal
            if LEGAL_MATCHER.match(sentence.lower().strip()):
                break

        if sentence:
            useful_sentences.append(sentence)
    return useful_sentences



def _strip_greeting(sentence):
    lowered_sentence = sentence.lower()
    greeting = _get_greeting(lowered_sentence)
    # strips from detected greeting to the closest comma
    # e.g 'hi anna,' -> ''
    # e.g 'hi andrew, how are you?' -> ' how are you'

    # TODO(hii): POS tagging also needed here
    # failure cases:
    # hello anna\n how are you?
    num_words = len(sentence.strip().split(" "))
    if greeting and ((',' in sentence) or num_words == 2):
        sentence = sentence[len(greeting):]
        return ','.join(sentence.split(',')[1:]).strip()
    return sentence

def _get_greeting(sentence):
    for g in constants.GREETINGS:
        if sentence.startswith(g):
            return g

def _contains_valid_farewell(sentence):
    valid_farewell = False
    # for multi lingual emails, it is common to split farewell by /
    for variation in re.split(r'/|\|', sentence):
        variation = variation.strip()
        farewell = _get_farewell(variation.lower())
        if farewell and len(variation.split(" ")) <= (len(farewell.split(" ")) + NAME_ALLOWANCE):
            valid_farewell = True
            break
    return valid_farewell

def _get_farewell(sentence):
    result = FAREWELL_MATCHER.search(sentence.strip())
    if result:
        return result.group(0)



"""
Cleans email bodies. Can remove html, greetings and split to sentences

Use `html_to_sentences` to split html input
"""
import contractions
import html
import re
import unicodedata

from email_reply_parser import EmailReplyParser
from bs4 import BeautifulSoup, Comment

from neon_goby import constants

def clean(text, anonymous=True):
    """
    Removes html, cleans.
    """
    text = html.unescape(text)
    text = unicodedata.normalize('NFKD', text)

    # handle outlook quotes
    text = _remove_with(text, constants.HTML_QUOTES, '')

    text = strip_html(text)
    text = _remove_quoted_email(text)
    text = _remove_empty_lines(text)

    if anonymous:
        text = replace_entities(text)

    text = contractions.fix(text)

    return text

def strip_html(text):
    soup = BeautifulSoup(text, "html.parser")
    # only apply stripping if text is html
    if soup.find('html'):
        for quoted_div in soup.findAll('div', {"class": "gmail_quote"}):
            quoted_div.decompose()

        for style_section in soup.findAll('style'):
            style_section.decompose()

        text = soup.get_text()

    return text

def _remove_with(text, regexes, replacement):
    for r in regexes:
        text = r.sub(replacement, text)
    return text

def replace_entities(text):
    for pattern, replacement in constants.ENTITIES:
        text = pattern.sub(replacement, text)
    return text

def _remove_quoted_email(string):
    string = _remove_with(string, constants.PREPENDED_TEXT, '')
    string = EmailReplyParser.read(string).reply
    for quote_pattern in constants.QUOTED_EMAILS:
        if re.match(quote_pattern, string):
            string = re.split(quote_pattern, string)[0]
    return string

def _remove_empty_lines(text):
    return '\n'.join([s for s in text.splitlines() if s.strip()])



import re

# Plain email format
PREPENDED_TEXT = [
    re.compile(r"^Sender [^\r\n]+\r\nTo: [^\r\n]+\r\n(?:CC: .*\r\n)?")
]

# Plain text quote indicators
QUOTED_EMAILS = [
    re.compile(r"On .+, <.+@.+\.\w+> wrote:"),
    re.compile(r"From: .+ <.+@.+\.\w+>"),
    re.compile(r"On .+, [^\n\r]+ wrote:"),
    re.compile(r"From: .+[\n\r]+Sent: .+[\n\r]+To:", flags=re.MULTILINE),
    re.compile(r"From: .+[\n\r]+Date: .+[\n\r]+To:", flags=re.MULTILINE)
]

# Entity patterns
EMAIL_MATCHER = r"[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-\.]+\.[a-zA-Z0-9]+"

# Outlook quote tags
HTML_QUOTES = [
    re.compile(r"(?ms)<[^>]*appendonsend.*"),
    re.compile(r"(?ms)<[^>]*[dD]iv[rR]ply[fF]wd[mM]sg.*"),
]

PHONE_MATCHERS = "|".join([
    r"\+?\(?\d{2}\)? ?\d{4} ?\d{4}",
    r"\d{4} ?\d{4}",
    r"\+?\(?\d{2}\)? ?\d{4} ?\d{4}",
    r"\+?\d{2} ?\(?\d{2,4}\)? ?\d{2,4} ?\-?\d{2,4} ?\d{0,2}"
])

WEB_MATCHERS = "|".join([
    r"http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+",
    r"[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b([-a-zA-Z0-9()@:%_\+.~#?&//=]*)"
])

ENTITIES = [
    [re.compile(EMAIL_MATCHER), ' <email> '],
    [re.compile(PHONE_MATCHERS), ' <phone> '],
    [re.compile(WEB_MATCHERS), ' <web> '],
]

REPLACEMENTS = [
    [re.compile(r"\d"), '9'], # Replace all digits
]

SPACES = [re.compile(r"[ \t]+")]

GREETINGS = [
    'dear', 'hi', 'hello', 'greetings', 'to whom', 'for whom', 'good', 'hey'
]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="neon-goby", # Replace with your own username
    version="0.0.4",
    author="HiiYL",
    author_email="hii@saleswhale.com",
    description="A simple package to clean email bodies",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(exclude=['tests']),
    install_requires=[
        'contractions==0.0.23',
        'email_reply_parser==0.5.9',
        'syntok==1.2.1',
        'beautifulsoup4==4.7.1'
    ],
    test_requires=[
        'pytest',
        'pytest-cov',
        'pylint'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
